
#include <interrupt.h>
#include <mzlib.h>
#include <string.h>
#include <stdio.h>
#include "ArkosTrackerPlayer_MZ.h"

#include "kuvat.h"
#include "zoomer.h"
#include "balkkar_osa.h"
#include "blind_osa.h"
#include "ympyraz_osa.h"
#include "pano_osa.h"
#include "greets.h"

#define SHOW_INITSCREEN

static unsigned char bgcols[40];

extern unsigned char marx[];
extern unsigned char chaplin[];
extern unsigned char csharp[];

// this sine table is used by multiple parts
signed char sini[256]={
    0,3,6,9,12,15,18,21,24,27,30,33,36,39,42,45,48,51,54,57,59,62,65,67,70,73,75,
    78,80,82,85,87,89,91,94,96,98,100,102,103,105,107,108,110,112,113,114,116,117,
    118,119,120,121,122,123,123,124,125,125,126,126,126,126,126,127,126,126,126,
    126,126,125,125,124,123,123,122,121,120,119,118,117,116,114,113,112,110,108,
    107,105,103,102,100,98,96,94,91,89,87,85,82,80,78,75,73,70,67,65,62,59,57,54,
    51,48,45,42,39,36,33,30,27,24,21,18,15,12,9,6,3,0,-3,-6,-9,-12,-15,-18,-21,
    -24,-27,-30,-33,-36,-39,-42,-45,-48,-51,-54,-57,-59,-62,-65,-67,-70,-73,-75,
    -78,-80,-82,-85,-87,-89,-91,-94,-96,-98,-100,-102,-103,-105,-107,-108,-110,
    -112,-113,-114,-116,-117,-118,-119,-120,-121,-122,-123,-123,-124,-125,-125,
    -126,-126,-126,-126,-126,-127,-126,-126,-126,-126,-126,-125,-125,-124,-123,
    -123,-122,-121,-120,-119,-118,-117,-116,-114,-113,-112,-110,-108,-107,-105,
    -103,-102,-100,-98,-96,-94,-91,-89,-87,-85,-82,-80,-78,-75,-73,-70,-67,-65,
    -62,-59,-57,-54,-51,-48,-45,-42,-39,-36,-33,-30,-27,-24,-21,-18,-15,-12,-9,
    -6,-3};


volatile unsigned char kount=0;
volatile unsigned char musa_tick=0; // 0..17
volatile unsigned char musa_beat=0; // 0..3
volatile unsigned char musa_bar=0; // 0..255


void my_isr(void) interrupt __naked
{
    _asm

    
    push    af
    push    hl
    push    bc
    push    de
    push    ix
    push    iy
    
    
    ld      hl,#0xe006 ; Kello laskemaan
    ld      a,#1
    ld      (hl),a
    xor     a
    ld      (hl),a        
    
    ld      a, (_kount)
    inc     a
    ld      (_kount), a
    
    ld      a, (_musa_tick)
    inc     a
    cp      #18
    jr      nz, 4$
    
    ; tick needs to be zeroed
    
    ld      a, (_musa_beat)
    inc     a
    cp      #4
    jr      nz, 5$
    
    ; eighth needs to be zeroed
    
    ld      a, (_musa_bar)
    inc     a
    ld      (_musa_bar), a
    
    xor     a
    
5$:
    ld      (_musa_beat), a

    xor     a
4$: 
    ld      (_musa_tick), a
    
    
    ld      hl,#0xe004
    ld      a, (_PLY_PSGRegistersArray)
    ld      b, a
    sla     a
    sla     a
    sla     a
    sla     a
    ld      (hl), a
    srl     b
    srl     b
    srl     b
    srl     b
    ld      a, (_PLY_PSGRegistersArray+1) 
    sla     a
    sla     a
    sla     a
    sla     a
    add     a, b
    ld      (hl), a
    
    ;    ld      a, (_PLY_PSGReg8) 
    ;or      a
    ;jr      z, 1$
    
    ;ld      hl,#0xe008
    ;ld      (hl),#1
    ;jr      2$
    
    1$:
    ;    ld hl,#0xe008
    ;ld (hl),#0
    
    2$:
    ;ld hl,#0xe008
    ;ld (hl),#0
    
    call    _PLY_Play

    pop     iy
    pop     ix
    pop     de
    pop     bc
    pop     hl
    pop     af

    ei
    
    reti
    _endasm;
}

void shut_up()
{
    poke(0xe008, 0); // ni pois    
    uninstall_isr();
}

extern void initscreen(void);

void main(void)
{
    static unsigned char prevvi=0;
    static unsigned char koko=0;
    static unsigned char *p, *p2, *srcptr1, *srcptr2, *dstptr, *dstptr2;
    static unsigned char ang, amountti;
    static unsigned char c=0,draw=20,red=20,cl=0,n;
    static unsigned char *colorfadeptr;
    static unsigned char suunta;

    int writecount=0;

    // initialize Arkos Tracker Player routine
    PLY_SongPtr = (char *)0x1203;
    PLY_Init();
    
#ifdef SHOW_INITSCREEN 
    // alkuruutu-writer
    initscreen();
#endif
    
    // title screen 
    mem_cpy((char *)CHAR_TABLE,csharp,1000);
    mem_cpy((char *)COLOR_TABLE,csharp+1000,1000);
    for(n=0;n<125;n++)
        waitVB();

    // fade out title screen
    dstptr = (void *)COLOR_TABLE;
    dstptr2 = (char *)COLOR_TABLE + 920;
    for (n=0; n<14; n++)
    {
        waitVB();
        if (dstptr < (COLOR_TABLE + 1000)) 
        {
            mem_set(dstptr, 0x00, 40);
            dstptr += 80;
        }
        waitVB();
        if (dstptr2 >= COLOR_TABLE) 
        {
            mem_set(dstptr2, 0x00, 40);
            dstptr2 -= 80;
        }
    }
    
    kuvat_init();
    zoomer_init();
    waitVB();

    // MUSIIKKI ALKAA NYTEN!

    install_isr(my_isr);
    
    DI;
    poke(0xe007,0xb0); // Laskuri 2 laskee yhden kerran
    poke(0xe006,1);
    poke(0xe006,0);

    poke(0xe007,0x74); // 1 tykitt sataa Hertsi
    poke(0xe005,156 & 0xff);
    poke(0xe005,0);
    
    poke(0xe008, 1); // ni plle
    EI;


    // vrien alustus
    mem_set((void *)COLOR_TABLE, BG_BLUE + BLACK, 1000);
    colorfadeptr = COLOR_TABLE;
    
    // ******************************************************************* ZUUMI
    while(!space() && (musa_bar < 14))
    {
        while(prevvi==kount)
            ;
        
        p = kuvat[musa_beat];
        if ((musa_bar >= 6) && (musa_bar < 7))
            // tss iso synkkapiste, bassoliuku 6:1
        {
            if (musa_beat >= 1) koko = (4 - musa_beat);
            zoomer_do(0, p, koko, (void *)CHAR_TABLE, 0);
        }
        else if ((musa_bar >= 7) && (musa_bar < 8))
            zoomer_do(1, p, 3, (void *)CHAR_TABLE, 0);
        else if (musa_bar < 6)
        {
            if (musa_tick <= 1)
                koko = (3 - musa_beat);
                
            zoomer_do(0, p, koko, (void *)CHAR_TABLE, 0);
        }
        else
        {
            if ((musa_bar == 8) && (musa_beat == 0) && (musa_tick <= 1))
            {
                // vrien alustus
                mem_set((void *)COLOR_TABLE, BG_RED + WHITE, 1000);
            }
            
            if (musa_bar <= 12)
                zoomer_do(0, p, 3, (void *)CHAR_TABLE, 0);
            else
            if (musa_bar >= 13)
            {                
                if (colorfadeptr < COLOR_TABLE + 1000)
                {
                    zoomer_do(0, p, 3, (void *)CHAR_TABLE, 0);
                    mem_set(colorfadeptr, 0x00, 40);
                    colorfadeptr += 40;
                } 
                else 
                {
                    mem_set((char *)CHAR_TABLE, 0x00, 1000);
                }

            }
        }
        if (koko > 0) koko--;
        
        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }
    
    // ******************************************************************* YMPYRA
    ang=0; amountti=0;
    waitVB();
    init_ympyraz();
    colorfadeptr = (char *)COLOR_TABLE;
    while(!space() && (musa_bar < 24))
    {
        while(prevvi==kount)
            ;
        
        run_ympyraz(ang,amountti);
        
        ang+=2; // Increase of 2 = one step
        
        if((ang&127) == 0)
            amountti++;

        if (musa_bar >= 23)
        {                
            if (colorfadeptr < COLOR_TABLE + 1000)
            {
                mem_set(colorfadeptr, 0x00, 40);
                colorfadeptr += 40;
            } 
            else 
            {
                mem_set((char *)CHAR_TABLE, 0x00, 1000);
            }            
        }
        
        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }



    // ******************************************************************* CREDITS-KUVA 1
    mem_set((char *)COLOR_TABLE, 0x00, 1000);
    waitVB();
    mem_cpy((void *)CHAR_TABLE, chaplin, 1000);
    p = chaplin + 1000;    
    p2 = chaplin + 1920;    
    dstptr = (void *)COLOR_TABLE;
    dstptr2 = (char *)COLOR_TABLE + 920;
    srcptr1 = chaplin + 1000;
    srcptr2 = srcptr1;
    for (n=0; n<14; n++)
    {
        while(prevvi==kount) ; prevvi = kount;
        if (dstptr < (COLOR_TABLE + 1000)) 
        {
            mem_cpy(dstptr, p, 40);
            dstptr += 80;
            p += 80;
        }
        while(prevvi==kount) ; prevvi = kount;
        if (dstptr2 >= COLOR_TABLE) 
        {
            mem_cpy(dstptr2, p2, 40);
            dstptr2 -= 80;
            p2 -= 80;
        }
    }
    while(!space() && (musa_bar < 28))
    {
        while(prevvi==kount)
            ; 

        if (musa_bar >= 27)
            zoomer_do(0, p, 3 - musa_beat, (void *)COLOR_TABLE, 1);
        else if (musa_bar == 26)
        {
            if (srcptr1 < chaplin + 1000)
            {
                mem_cpy(buf1, srcptr1, 40);
                srcptr1 += 40;
            }
        }
        else if (musa_bar == 25)
        {
            if (srcptr2 < chaplin + 1000)
            {
                mem_cpy(buf2, srcptr2, 40);
                srcptr2 += 40;
            }
        }
//        else
//            mem_cpy((void *)COLOR_TABLE, chaplin + 1000, 1000);
        
        prevvi=kount; 
    }
    if (space())
    {
        shut_up();
        return;
    }


    // ******************************************************************* CREDITS-KUVA 2
    mem_set((char *)COLOR_TABLE, 0x00, 1000);
    waitVB();
    mem_cpy((void *)CHAR_TABLE, marx, 1000);
    p = marx + 1000;    
    p2 = marx + 1920;    
    dstptr = (char *)COLOR_TABLE;
    dstptr2 = (char *)COLOR_TABLE + 920;
    srcptr1 = marx + 1000;
    srcptr2 = srcptr1;
    for (n=0; n<14; n++)
    {
        while(prevvi==kount) ; prevvi = kount;
        if (dstptr < (COLOR_TABLE + 1000)) 
        {
            mem_cpy(dstptr, p, 40);
            dstptr += 80;
            p += 80;
        }
        while(prevvi==kount) ; prevvi = kount;
        if (dstptr2 >= COLOR_TABLE) 
        {
            mem_cpy(dstptr2, p2, 40);
            dstptr2 -= 80;
            p2 -= 80;
        }
    }
    while(!space() && (musa_bar < 32))
    {
        while(prevvi==kount)
            ;
        
        if (musa_bar >= 31)
            zoomer_do(0, p, 3 - musa_beat, (void *)COLOR_TABLE, 1);
        else if (musa_bar == 30)
        {
            if (srcptr1 < marx + 1000)
            {
                mem_cpy(buf1, srcptr1, 40);
                srcptr1 += 40;
            }
        }
        else if (musa_bar == 29)
        {
            if (srcptr2 < marx + 1000)
            {
                mem_cpy(buf2, srcptr2, 40);
                srcptr2 += 40;
            }
        }
        
        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }
    
    
    // ******************************************************************* PALKIT
    init_balkkar();
    while(!space() && (musa_bar < 40))
    {
        while(prevvi==kount)
            ;

        if (musa_bar < 49)
        {
            if (musa_beat == 3)
            {
                n = kount; // have to use temp variable to circumvent sdcc bug
                if (musa_tick < 3)
                    run_balkkar(8, 0x11, n);
                else
                    run_balkkar(8, 0x00, n);
            }
            else if ((musa_beat == 2) && (musa_tick < 3))
            {
                n = kount;
                if (musa_tick < 3)
                    run_balkkar(8, 0x77, n);
                else
                    run_balkkar(8, 0x00, n);
            }
            else
                run_balkkar(8, 0x00, kount);  
        }
        else
        {
            if ((musa_beat == 3) && (musa_tick < 3))
            {
                n = kount;
                if (musa_tick < 3)
                    run_balkkar(6, 0x11, n);
                else
                    run_balkkar(6, 0x00, n);
            }
            else if ((musa_beat == 2) && (musa_tick < 3))
            {
                n = kount;
                if (musa_tick < 3)
                    run_balkkar(6 - (musa_beat << 1), 0x77, n);
                else
                    run_balkkar(6 - (musa_beat << 1), 0x00, n);
            }
            else
                run_balkkar(6 - (musa_beat << 1), 0x00, kount);  
        }

        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }
    
    waitVB();
    mem_set((void *)COLOR_TABLE, 0, 1000);
    waitVB();
    mem_set((void *)CHAR_TABLE, 0, 1000);
    
    // ******************************************************************* BLIND
    init_blind(0x70, 0x20);
    init_greet();
    while(!space() && (musa_bar < 50))
    {
        while(prevvi==kount)
            ;
        
        run_blind(kount);
        run_greet(writecount);
        
        prevvi=kount;
        writecount++;
    }
    if (space())
    {
        shut_up();
        return;
    }

    // ******************************************************************* BREAK 2!
    while(!space() && (musa_bar < 52))
    {
        while(prevvi==kount)
            ;
        
        if ((musa_bar >= 50) && (musa_bar < 51))
        {
            p = kuvat[3 - musa_beat];
            // tss iso synkkapiste, bassoliuku 6:1
            zoomer_do((musa_tick <= 1), p, 3, (void *)CHAR_TABLE, 0);
        }
        else if ((musa_bar >= 51) && (musa_bar < 52))
        {
            p = kuvat[musa_beat];
            if ((musa_beat == 3) && (musa_tick > 3))
            {
                    zoomer_do(0, p, 0, (void *)COLOR_TABLE, 1);
                    zoomer_do(0, p, 0, (void *)CHAR_TABLE, 0);
            }
            /*                else if (musa_tick > 3) 
                    zoomer_do(1, p, 0, (void *)COLOR_TABLE, 1);
                else
                    zoomer_do(1, p, 0, (void *)CHAR_TABLE, 0);
            } */
            else
                zoomer_do(1, p, 3, (void *)CHAR_TABLE, 0);
        } 
        else 
            zoomer_do(0, p, 1, (void *)CHAR_TABLE, 0);
        
        
        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }

    while(!space() && (musa_bar < 53))
    {
        while(prevvi==kount)
            ;
        
        zoomer_do(0, p, 0, (void *)COLOR_TABLE, 1);
        zoomer_do(0, p, 0, (void *)CHAR_TABLE, 1);
        
        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }
    
    // ******************************************************************* PANO
    suunta=1;
    c=0; draw=20; red=20; cl=0;
    for(n=0;n<40;n++)
        bgcols[n]=0x80+0x20;

    init_pano(ekakuva);
    while(!space() && (musa_bar < 75))
    {
        while(prevvi==kount)
            ;
        
        if ((musa_bar < 73))
        {
            run_pano(c);
            c += suunta;

            if ((musa_bar < 56))
            {            
                bgcols[draw]=0x80+0x60;
                bgcols[cl]=0x80+0x20;
                bgcols[red]=0x80+0x20;
            }
            else if ((musa_bar >= 70) && (musa_beat == 1))
            {            
                bgcols[draw]=0x80+0x63;
                bgcols[cl]=0x80+0x03;
                bgcols[red]=0x80+0x23;
                suunta = -2;
            }
            else if ((musa_bar >= 60) && (musa_beat == 1))
            {            
                bgcols[draw]=0x80+0x64;
                bgcols[cl]=0x80+0x04;
                bgcols[red]=0x80+0x24;
                suunta = -2;
            }
            else if ((musa_bar >= 56) && (musa_beat == 1))
            {            
                bgcols[draw]=0x80+0x65;
                bgcols[cl]=0x80+0x75;
                bgcols[red]=0x80+0x25;
                suunta = -1;
            }
            else
            {            
                bgcols[draw]=0x80+0x60;
                bgcols[cl]=0x80+0x00;
                bgcols[red]=0x80+0x20;
                suunta = 1;
            }
            
            bgcolumns(bgcols);
            
        } 
        else 
        {
            run_pano(c);
            c++;

            if ((musa_bar >= 74) && (musa_beat == 1) && (musa_tick < 10))
            {
                bgcols[draw]=0x77;
                bgcols[cl]=0x77;
                bgcols[red]=0x77;
            }
            else 
            {
                bgcols[draw]=0x00;
                bgcols[cl]=0x00;
                bgcols[red]=0x00;
            }
            
            bgcolumns(bgcols);
        }

        draw=(draw+1)%40; // Modulot poies herrajjumala!
        cl=(cl+(musa_beat!=2))%40;
        red--;
        if(red==255)
            red=39;
        
        prevvi=kount;
    }
    if (space())
    {
        shut_up();
        return;
    }
    
    mem_set((char *)COLOR_TABLE,0,40*25);
    shut_up();
    
//    download(); // Voidaan hypt Marqun siirtosoftaan takas

    // Jos tlt mennn normaalisti ulos, niin palataan monitoriin
}
